import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:path_provider/path_provider.dart';

class ApiService {
  late Dio _dio;
  late String _baseUrl;

  ApiService(Dio dio, String baseUrl) {
    _dio = dio;
    _baseUrl = baseUrl;
  }

  Future<Response> get(
    String path, {
    Map<String, dynamic>? queryParameters,
  }) =>
      _dio.get(_baseUrl + path, queryParameters: queryParameters);

  Future<Response> postRequest(String path, dynamic data) =>
      _dio.post(_baseUrl + path, data: data);

  Future<Response> putRequest(String path, dynamic data) =>
      _dio.put(_baseUrl + path, data: data);

  Future<Response> deleteRequest(String path) => _dio.delete(_baseUrl + path);

  Future<Stream<List<int>>> getStreamResponse(
    String path, {
    Map<String, dynamic>? queryParameters,
  }) async {
    try {
      final response = await _dio.get(
        _baseUrl + path,
        queryParameters: queryParameters,
        options: Options(responseType: ResponseType.stream),
      );
      return response.data.stream;
    } catch (e) {
      rethrow;
    }
  }

  Future<Response> postStreamRequest(
    String path,
    Stream<List<int>> stream,
  ) =>
      _dio.post(
        _baseUrl + path,
        data: stream,
        options: Options(
          contentType: Headers.formUrlEncodedContentType,
        ),
      );

  Future<Response> putStreamRequest(
    String path,
    Stream<List<int>> stream,
  ) =>
      _dio.put(
        _baseUrl + path,
        data: stream,
        options: Options(
          contentType: Headers.formUrlEncodedContentType,
        ),
      );

  Future<Response> postStreamRequestWithProgress(
    String path,
    Stream<List<int>> dataStream,
    Function(int, int) onSendProgress,
  ) =>
      _dio.post(
        _baseUrl + path,
        data: dataStream,
        options: Options(
          responseType: ResponseType.bytes,
          headers: {
            'Content-Type': 'application/octet-stream',
          },
        ),
        onSendProgress: onSendProgress,
      );

  Future<List<int>?> getBytesResponse(
    String path, {
    Map<String, dynamic>? queryParameters,
  }) async {
    try {
      final response = await _dio.get<List<int>>(
        _baseUrl + path,
        queryParameters: queryParameters,
        options: Options(responseType: ResponseType.bytes),
      );
      return response.data;
    } catch (e) {
      rethrow;
    }
  }

  Future<Response> postBytesRequest(
    String path,
    List<int> data,
  ) =>
      _dio.post(
        _baseUrl + path,
        data: data,
        options: Options(
          responseType: ResponseType.bytes,
        ),
      );

  Future<Response> putBytesRequest(
    String path,
    List<int> data,
  ) =>
      _dio.put(
        _baseUrl + path,
        data: data,
        options: Options(
          responseType: ResponseType.bytes,
        ),
      );

  Future<Response> postBytesRequestWithProgress(
    String path,
    List<int> data,
    Function(int, int) onSendProgress,
  ) =>
      _dio.post(
        _baseUrl + path,
        data: Stream.fromIterable(data),
        options: Options(
          responseType: ResponseType.bytes,
          headers: {
            'Content-Type': 'application/octet-stream',
          },
        ),
        onSendProgress: onSendProgress,
      );

  Future<Response> downloadFile(
    String path,
    String file,
  ) =>
      _dio.download(
        _baseUrl + path,
        (Headers headers) async {
          // Extra info: redirect counts
          debugPrint('downloadFile: ${headers.value('redirects')}');
          // Extra info: real uri
          debugPrint('downloadFile: ${headers.value('uri')}');
          // ...
          return '${(await getTemporaryDirectory()).path}$file';
        },
        // options: Options(
        //   headers: {HttpHeaders.acceptEncodingHeader: '*'}, // Disable gzip
        // ),
        onReceiveProgress: (received, total) {
          if (total != -1) {
            debugPrint(
                'downloadFile: ${(received / total * 100).toStringAsFixed(0)}%');
          }
        },
      );
}
