import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:image_picker/image_picker.dart';
import 'package:permission_handler/permission_handler.dart';

class ServiceControlCenter {
  const ServiceControlCenter();

  static final GlobalKey<NavigatorState> _navigatorKey =
      GlobalKey<NavigatorState>();

  static GlobalKey<NavigatorState> get navigatorKey => _navigatorKey;

  static final GlobalKey<ScaffoldMessengerState> _scaffoldMessengerKey =
      GlobalKey<ScaffoldMessengerState>();

  static GlobalKey<ScaffoldMessengerState> get scaffoldMessengerKey =>
      _scaffoldMessengerKey;

  static void navigateToPage(String routeName, {Object? arguments}) {
    _navigatorKey.currentState?.pushNamed(routeName, arguments: arguments);
  }

  static void navigateToReplacement(String routeName, {Object? arguments}) {
    _navigatorKey.currentState
        ?.pushReplacementNamed(routeName, arguments: arguments);
  }

  static void navigateAndRemoveUntil(String routeName, {Object? arguments}) {
    _navigatorKey.currentState?.pushNamedAndRemoveUntil(
        routeName, (route) => false,
        arguments: arguments);
  }

  static void navigateBack() {
    _navigatorKey.currentState?.pop();
  }

  static void navigateBackUntil(String routeName) {
    _navigatorKey.currentState?.popUntil(ModalRoute.withName(routeName));
  }

  // show snack bar
  static void showToast(
    String message, {
    TextStyle? textStyle,
    double radius = 8.0,
    Color backgroundColor = Colors.black,
    double elevation = 6,
    EdgeInsets? margin,
  }) =>
      _scaffoldMessengerKey.currentState?.showSnackBar(
        SnackBar(
          content: Text(
            message,
            textAlign: TextAlign.center,
            style: textStyle,
          ),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius),
          ),
          behavior: SnackBarBehavior.floating,
          elevation: elevation,
          margin: margin,
          backgroundColor: backgroundColor,
        ),
      );

  // hide snack bar
  static void hideToast() =>
      _scaffoldMessengerKey.currentState?.clearSnackBars();

  // show snack bar
  static void showToastWithContext(
    BuildContext context,
    String message, {
    TextStyle? textStyle,
    double radius = 8.0,
    Color backgroundColor = Colors.black,
    double elevation = 6,
    EdgeInsets? margin,
  }) =>
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Text(
            message,
            textAlign: TextAlign.center,
            style: textStyle,
          ),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius),
          ),
          behavior: SnackBarBehavior.floating,
          elevation: elevation,
          margin: margin,
          backgroundColor: backgroundColor,
        ),
      );

  // hide snack bar
  static void hideToastWithContext(BuildContext context) =>
      ScaffoldMessenger.of(context).clearSnackBars();

  // set color status bar
  static void setStatusBarColor(Color color) {
    SystemChrome.setSystemUIOverlayStyle(
      SystemUiOverlayStyle(statusBarColor: color),
    );
  }

  // reset status bar
  static void resetStatusBarColor() {
    SystemChrome.restoreSystemUIOverlays();
  }

  // Method to request a permission
  static Future<PermissionStatus> requestPermission(
          Permission permission) async =>
      await permission.request();

  // Method to check if a permission is granted
  static Future<bool> checkPermissionStatus(Permission permission) async {
    final status = await permission.status;
    return status.isGranted;
  }

  // Option for take a picture
  static Future<void> showImagePicker(
    BuildContext context,
    Function(File?) onImagePicked, {
    IconData? iconGallery,
    String? textSelectGallery,
    IconData? iconCamera,
    String? textSelectCamera,
  }) async =>
      await showModalBottomSheet(
        context: context,
        builder: (_) {
          return Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              ListTile(
                leading: Icon(iconGallery ?? Icons.photo_library),
                title: Text(textSelectGallery ?? 'Pick from Gallery'),
                onTap: () async {
                  Navigator.pop(context);
                  File? image = await getImage(ImageSource.gallery);
                  onImagePicked(image);
                },
              ),
              ListTile(
                leading: Icon(iconCamera ?? Icons.camera_alt),
                title: Text(textSelectCamera ?? 'Capture Image'),
                onTap: () async {
                  Navigator.pop(context);
                  File? image = await getImage(ImageSource.camera);
                  onImagePicked(image);
                },
              ),
            ],
          );
        },
      );

  /// Get image follow type selected
  /// 1. ImageSource.gallery
  /// 2. ImageSource.camera
  static Future<File?> getImage(ImageSource type) async {
    final ImagePicker picker = ImagePicker();
    final XFile? pickedFile = await picker.pickImage(source: type);
    if (pickedFile != null) {
      return File(pickedFile.path);
    }
    return null;
  }

  static bool isNullOrEmpty(String? value) {
    if (value == null || value.isEmpty) {
      return true;
    }
    return false;
  }
}
